@@ -8,6 +8,7 @@ use crate::{
     generate::generate_parser_for_grammar,
     parse::{perform_edit, Edit},
 };
+use proc_macro::retry;
 use std::{
     sync::atomic::{AtomicUsize, Ordering},
     thread, time,
@@ -505,7 +506,7 @@ fn test_parsing_after_detecting_error_in_the_middle_of_a_string_token() {
     let tree = parser.parse(&source, None).unwrap();
     assert_eq!(
         tree.root_node().to_sexp(),
-        "(module (expression_statement (assignment left: (identifier) right: (expression_list (identifier) (string)))))"
+        "(module (expression_statement (assignment left: (identifier) right: (expression_list (identifier) (string string_content: (string_content))))))"
     );
 
     // Delete a suffix of the source code, starting in the middle of the string
@@ -638,6 +639,7 @@ fn test_parsing_cancelled_by_another_thread() {
 // Timeouts
 
 #[test]
+#[retry(10)]
 fn test_parsing_with_a_timeout() {
     let mut parser = Parser::new();
     parser.set_language(get_language("json")).unwrap();
@@ -829,6 +831,7 @@ fn test_parsing_with_one_included_range() {
         js_tree.root_node().start_position(),
         Point::new(0, source_code.find("console").unwrap())
     );
+    assert_eq!(js_tree.included_ranges(), &[script_content_node.range()]);
 }
 
 #[test]
@@ -853,28 +856,27 @@ fn test_parsing_with_multiple_included_ranges() {
     let close_quote_node = template_string_node.child(3).unwrap();
 
     parser.set_language(get_language("html")).unwrap();
-    parser
-        .set_included_ranges(&[
-            Range {
-                start_byte: open_quote_node.end_byte(),
-                start_point: open_quote_node.end_position(),
-                end_byte: interpolation_node1.start_byte(),
-                end_point: interpolation_node1.start_position(),
-            },
-            Range {
-                start_byte: interpolation_node1.end_byte(),
-                start_point: interpolation_node1.end_position(),
-                end_byte: interpolation_node2.start_byte(),
-                end_point: interpolation_node2.start_position(),
-            },
-            Range {
-                start_byte: interpolation_node2.end_byte(),
-                start_point: interpolation_node2.end_position(),
-                end_byte: close_quote_node.start_byte(),
-                end_point: close_quote_node.start_position(),
-            },
-        ])
-        .unwrap();
+    let html_ranges = &[
+        Range {
+            start_byte: open_quote_node.end_byte(),
+            start_point: open_quote_node.end_position(),
+            end_byte: interpolation_node1.start_byte(),
+            end_point: interpolation_node1.start_position(),
+        },
+        Range {
+            start_byte: interpolation_node1.end_byte(),
+            start_point: interpolation_node1.end_position(),
+            end_byte: interpolation_node2.start_byte(),
+            end_point: interpolation_node2.start_position(),
+        },
+        Range {
+            start_byte: interpolation_node2.end_byte(),
+            start_point: interpolation_node2.end_position(),
+            end_byte: close_quote_node.start_byte(),
+            end_point: close_quote_node.start_position(),
+        },
+    ];
+    parser.set_included_ranges(html_ranges).unwrap();
     let html_tree = parser.parse(source_code, None).unwrap();
 
     assert_eq!(
@@ -888,6 +890,7 @@ fn test_parsing_with_multiple_included_ranges() {
             " (end_tag (tag_name))))",
         )
     );
+    assert_eq!(html_tree.included_ranges(), html_ranges);
 
     let div_element_node = html_tree.root_node().child(0).unwrap();
     let hello_text_node = div_element_node.child(1).unwrap();
@@ -950,7 +953,9 @@ fn test_parsing_with_included_range_containing_mismatched_positions() {
 
     parser.set_included_ranges(&[range_to_parse]).unwrap();
 
-    let html_tree = parser.parse(source_code, None).unwrap();
+    let html_tree = parser
+        .parse_with(&mut chunked_input(source_code, 3), None)
+        .unwrap();
 
     assert_eq!(html_tree.root_node().range(), range_to_parse);
 
@@ -1077,7 +1082,9 @@ fn test_parsing_with_a_newly_excluded_range() {
     // Parse HTML including the template directive, which will cause an error
     let mut parser = Parser::new();
     parser.set_language(get_language("html")).unwrap();
-    let mut first_tree = parser.parse(&source_code, None).unwrap();
+    let mut first_tree = parser
+        .parse_with(&mut chunked_input(&source_code, 3), None)
+        .unwrap();
 
     // Insert code at the beginning of the document.
     let prefix = "a very very long line of plain text. ";
@@ -1112,7 +1119,9 @@ fn test_parsing_with_a_newly_excluded_range() {
             },
         ])
         .unwrap();
-    let tree = parser.parse(&source_code, Some(&first_tree)).unwrap();
+    let tree = parser
+        .parse_with(&mut chunked_input(&source_code, 3), Some(&first_tree))
+        .unwrap();
 
     assert_eq!(
         tree.root_node().to_sexp(),
@@ -1163,7 +1172,9 @@ fn test_parsing_with_a_newly_included_range() {
     parser
         .set_included_ranges(&[simple_range(range1_start, range1_end)])
         .unwrap();
-    let tree = parser.parse(source_code, None).unwrap();
+    let tree = parser
+        .parse_with(&mut chunked_input(&source_code, 3), None)
+        .unwrap();
     assert_eq!(
         tree.root_node().to_sexp(),
         concat!(
@@ -1180,7 +1191,9 @@ fn test_parsing_with_a_newly_included_range() {
             simple_range(range3_start, range3_end),
         ])
         .unwrap();
-    let tree2 = parser.parse(&source_code, Some(&tree)).unwrap();
+    let tree2 = parser
+        .parse_with(&mut chunked_input(&source_code, 3), Some(&tree))
+        .unwrap();
     assert_eq!(
         tree2.root_node().to_sexp(),
         concat!(
@@ -1288,3 +1301,7 @@ fn simple_range(start: usize, end: usize) -> Range {
         end_point: Point::new(0, end),
     }
 }
+
+fn chunked_input<'a>(text: &'a str, size: usize) -> impl FnMut(usize, Point) -> &'a [u8] {
+    move |offset, _| text[offset..text.len().min(offset + size)].as_bytes()
+}
